########################### PSQF 6270 Example 5 using R #################################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("haven")) install.packages("haven")
library(haven) # To import SAS data with labels as table

#if (!require("expss")) install.packages("expss")
#library(expss) # To add variable and value labels, sorting

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("prediction")) install.packages("prediction")
library(prediction) # To get predicted values like Stata does

if (!require("nlme")) install.packages("nlme")
library(nlme) # To fit multivariate models without random intercepts

#if (!require("emmeans")) install.packages("emmeans")
#library(emmeans) # To get model-implied means in gls

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit path models 


######################################################################################
#####                  BEGIN DATA MANIPULATION OF EXAMPLE 5                      #####
#####                   CHANGE "filesave" TO YOUR FOLDER PATH                    #####
######################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/24_PSQF6270/PSQF6270_Example5/"
filename = "PSQF6270_Example5Wide.sas7bdat"
setwd(dir=filesave)

# Import Example 5 SAS data
Example5_wide = read_sas(data_file=paste0(filesave,filename)) 
# Convert to data frame without labels to use for analysis
Example5_wide = as.data.frame(Example5_wide)

# Stack into long format (one row per outcome per family)
Example5 = reshape(Example5_wide,  direction="long", idvar="FamilyID",
                   varying=c("KidMarital","MomMarital","DadMarital"),
                   v.names="marital", timevar="DVnum", times=c(1,2,3))

# Create per-person dummy codes
Example5$kid=0
Example5$mom=0
Example5$dad=0
Example5$kid[which(Example5$DVnum==1)]=1
Example5$mom[which(Example5$DVnum==2)]=1
Example5$dad[which(Example5$DVnum==3)]=1

# Remove missing predictors or row-specific outcome (will happen anyway)
Example5 = Example5[complete.cases(Example5[ ,
                    c("KidBoy","KidEd12","MomEd12","DadEd12","marital")]),]


######################################################################################
#####                   BEGIN EXAMPLE 5 PART 1 MODELS                            #####
######################################################################################

# Open external file to save results to
txtStart(file=paste0(filesave,"PSQF6270_Example5_R_Output_Part1mixed.txt"))

####### 2 Versions of Baseline Model 0 #######
print("R Empty Means, Unstructured Variance Models for Marital Attitudes")

print("R Model 0a: General Intercept (Dad=Ref DV) using 2 Dummy Codes")
Model0a = gls(data=Example5, method="REML", 
              model=marital~1+kid+mom,    # Fixed intercept will be for dad (as omitted)
              correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
              weights=varIdent(form=~1|DVnum))            # Separate variance by DV  
print("Print -2LL and Results"); -2*logLik(Model0a); summary(Model0a) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model0a, individual="3996");  
corMatrix(Model0a$modelStruct$corStruct)[[3]] # 3=rows/columns of R here

print("DF=2 Intercept Diff -- Get error that it used Chi-Square instead of F")
F0a = glht(model=Model0a, linfct=rbind(c(0,1,0),c(0,0,1)), df=139)
SaveF0a = summary(F0a, test=Ftest()); SaveF0a # Joint F-test
print("Get and show hidden results for F, dfnum, dfden, and p-value")
SaveF0a$test$fstat; SaveF0a$test$df; SaveF0a$df
pf(SaveF0a$test$fstat,df1=SaveF0a$test$df,df2=SaveF0a$df,lower.tail=FALSE)

print("Missing Intercepts and Difference -- Had to give it correct Denominator DF")
summary(glht(model=Model0a, df=139, linfct=rbind(
  "Kid Intercept (Dad+Diff)"    = c(1,1,0),  # in order of fixed effects
  "Mom Intercept (Dad+Diff)"    = c(1,0,1),
  "Kid vs. Mom: Intercept Diff" = c(0,-1,1))), test=adjusted("none"))

txtComment(" ") # insert blank space in output 


############################################################################################
print("R Model 0b: DV-Specific Intercepts using All 3 Dummy Codes")
Model0b = gls(data=Example5, method="REML", 
              model=marital~0+kid+mom+dad,                # 0 removes fixed intercept
              correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
              weights=varIdent(form=~1|DVnum))            # Separate variance by DV  
print("Print -2LL and Results")  # Btw, AIC and BIC are incorrect (match STATA)
-2*logLik(Model0b); summary(Model0b) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model0b, individual="3996") # R matrix = variances and covariances across outcomes
corMatrix(Model0b$modelStruct$corStruct)[[3]] # 3=rows/columns of R here

print("DF=2 Intercept Diff -- Get error that it used Chi-Square instead of F")
F0c = glht(model=Model0b, linfct=rbind(c(-1,1,0),c(0,-1,1)), df=139)
SaveF0c = summary(F0c, test=Ftest()); SaveF0a  # Joint F-test
print("Get and show hidden results for F, dfnum, dfden, and p-value")
SaveF0c$test$fstat; SaveF0c$test$df; SaveF0c$df
pf(SaveF0c$test$fstat,df1=SaveF0c$test$df,df2=SaveF0c$df,lower.tail=FALSE)

print("Pairwise Intercept Diffs -- Had to give it correct Denominator DF")
summary(glht(model=Model0b, df=139, linfct=rbind(
  "Kid vs. Mom: Intercept Diff" = c(-1,1,0),  # in order of fixed effects
  "Kid vs. Dad: Intercept Diff" = c(-1,0,1),
  "Mom vs. Dad: Intercept Diff" = c(0,-1,1))), test=adjusted("none"))

txtComment(" ") # insert blank space in output 


############################################################################################
print("R Model 1: DV-Specific Intercepts -- Add Kid Gender")
Model1 = gls(data=Example5, method="REML", 
             model=marital~0+kid+mom+dad+kid:KidBoy+mom:KidBoy+dad:KidBoy,
             correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
             weights=varIdent(form=~1|DVnum))            # Separate variance by DV  
print("Print -2LL and Results"); -2*logLik(Model1); summary(Model1) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model1, individual="3996"); corMatrix(Model1$modelStruct$corStruct)[[3]]

print("DF=2 Diff in KidBoy Effect -- Get error that it used Chi-Square instead of F")
F1 = glht(model=Model1, linfct=rbind(c(0,0,0,-1,1,0),c(0,0,0,-1,0,1)), df=138)
SaveF1 = summary(F1, test=Ftest()); SaveF0a  # Joint F-test
print("Get and show hidden results for F, dfnum, dfden, and p-value")
SaveF1$test$fstat; SaveF1$test$df; SaveF1$df
pf(SaveF1$test$fstat,df1=SaveF1$test$df,df2=SaveF1$df,lower.tail=FALSE)

print("KidBoy Slope Diffs -- Had to give it correct Denominator DF")
summary(glht(model=Model1, df=138, linfct=rbind(
  "Kid vs. Mom: KidBoy Slope Diff"     = c(0,0,0,-1,1,0),  # in order of fixed effects
  "Kid vs. Dad: KidBoy Slope Diff"     = c(0,0,0,-1,0,1),
  "Mom vs. Dad: KidBoy Slope Diff"     = c(0,0,0,0,-1,1),
  "Parent KidBoy Effect"               = c(0,0,0,0,1/2,1/2),
  "Kids vs. Parent KidBoy Effect Diff" = c(0,0,0,-1,1/2,1/2))), test=adjusted("none"))

txtComment(" ") # insert blank space in output 


############################################################################################
print("R Model 2: DV-Specific Intercepts -- KidBoy on Kid Only, Add Own Educ")
Model2 = gls(data=Example5, method="REML", 
             model=marital~0+kid+mom+dad +kid:KidBoy +kid:KidEd12+mom:MomEd12+dad:DadEd12,
             correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
             weights=varIdent(form=~1|DVnum))            # Separate variance by DV  
print("Print -2LL and Results"); -2*logLik(Model2); summary(Model2) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model2, individual="3996"); corMatrix(Model2$modelStruct$corStruct)[[3]] 

txtComment(" ") # insert blank space in output


############################################################################################
print("R Model 3: DV-Specific Intercepts -- Add Dad Educ (Control for Own Educ)")
Model3 = gls(data=Example5, method="REML", 
             model=marital~0+kid+mom+dad+ kid:KidBoy +kid:KidEd12+mom:MomEd12+dad:DadEd12
                           +kid:DadEd12+mom:DadEd12,
             correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
             weights=varIdent(form=~1|DVnum))            # Separate variance by DV  
print("Print -2LL and Results"); -2*logLik(Model3); summary(Model3) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model3, individual="3996"); corMatrix(Model3$modelStruct$corStruct)[[3]] 

print("DadEd Slope Diffs -- Had to give it correct Denominator DF")
summary(glht(model=Model3, df=136, linfct=rbind(
  "Kid vs. Mom: DadEd12 Slope Diff" = c(0,0,0,0,0,0,0,-1,1),  # in order of fixed effects
  "Kid vs. Dad: DadEd12 Slope Diff" = c(0,0,0,0,0,0,1,-1,0),
  "Mom vs. Dad: DadEd12 Slope Diff" = c(0,0,0,0,0,0,1,0,-1))), test=adjusted("none"))

print("Save yhat from fixed effects and Pearson residuals")
Example5$Model3pred = predict(Model3, type="response")
Example5$Model3res = residuals(Model3, type="pearson")
print ("Histogram of Residuals for normality")
hist(x=Example5$Model3res, freq=FALSE, ylab="Density",xlab="Model 3 Residuals") 
print ("Scatterplot of residuals by prediced for constant variance")
plot(x=Example5$Model3res, y=Example5$Model3pred,
     ylab="Residual",xlab="Model 3 Predicted Outcome") 

txtComment(" ") # insert blank space in output 

# Close external output file
txtStop()


############################################################################################
# Open external file to save results to
txtStart(file=paste0(filesave,"PSQF6270_Example5_R_Output_Part2mixed.txt"))

print("R Model 4: DV-Specific Intercepts -- Add Mom Educ (Controlling for Own+Dad Educ)")
print("To match path model, switch to ML estimation, but robust SEs not directly available")
Model4 = gls(data=Example5, method="ML", 
             model=marital~0+kid+mom+dad+ kid:KidBoy +kid:KidEd12+mom:MomEd12+dad:DadEd12
             +kid:DadEd12+mom:DadEd12 +kid:MomEd12+dad:MomEd12,
             correlation=corSymm(form=~DVnum|FamilyID),  # Unstructured correlations
             weights=varIdent(form=~1|DVnum))            # Separate variance by DV  
print("Print -2LL and Results"); -2*logLik(Model4); summary(Model4) 
print("Show R and RCORR matrices for first family in the data") 
getVarCov(Model4, individual="3996"); corMatrix(Model4$modelStruct$corStruct)[[3]]

print("DadEd Slope Diffs -- Had to give it correct Denominator DF")
summary(glht(model=Model4, df=135, linfct=rbind(
  "Kid vs. Mom: DadEd12 Slope Diff" = c(0,0,0,0,0,0,0,-1,1,0,0),  # in order of fixed effects
  "Kid vs. Dad: DadEd12 Slope Diff" = c(0,0,0,0,0,0,1,-1,0,0,0),
  "Mom vs. Dad: DadEd12 Slope Diff" = c(0,0,0,0,0,0,1,0,-1,0,0),
  "Kid vs. Mom: MomEd12 Slope Diff" = c(0,0,0,0,0,1,0,0,0,-1,0),  
  "Kid vs. Dad: MomEd12 Slope Diff" = c(0,0,0,0,0,0,0,0,0,-1,1),
  "Mom vs. Dad: MomEd12 Slope Diff" = c(0,0,0,0,0,-1,0,0,0,0,1))), test=adjusted("none"))

print("Save predicted marital attitudes and correlate with actual marital attitudes") 
Example5$Pred = predict(Model4, type="response")

rPred1 = cor.test(x=Example5$Pred[which(Example5$DVnum==1)], 
                  y=Example5$marital[which(Example5$DVnum==1)], method="pearson")
print("R and R2 for DV=1 Kid"); rPred1$estimate; rPred1$estimate^2

rPred2 = cor.test(x=Example5$Pred[which(Example5$DVnum==2)], 
                  y=Example5$marital[which(Example5$DVnum==2)], method="pearson")
print("R and R2 for DV=2 Mom"); rPred2$estimate; rPred2$estimate^2

rPred3 = cor.test(x=Example5$Pred[which(Example5$DVnum==3)], 
                  y=Example5$marital[which(Example5$DVnum==3)], method="pearson")
print("R and R2 for DV=3 Dad"); rPred3$estimate; rPred3$estimate^2


# Close external output file
txtStop()


######################################################################################
#####                   BEGIN EXAMPLE 5 PART 2 MODELS                            #####
######################################################################################

# Example of how to export a .csv file for use in Mplus
# Copy data, replace all missing values with -999 for Mplus
Example5_Mplus = Example5_wide
Example5_Mplus[is.na(Example5_Mplus)] <- -999

# Write to .csv file without column names
write.table(x=Example5_Mplus, col.names=FALSE, row.names=FALSE, sep=",",
            file=paste0(filesave,"PSQF6270_Example5Wide_R.csv"))
          
# Open external file to save results to
txtStart(file=paste0(filesave,"PSQF6270_Example5_R_Output_Part2path.txt"))

print("R Model 3: Own Education + Dad Education a Predictor of Each Attitude")

# Create model syntax as separate text object
Syntax3 = "
# Residual variances estimated separately (by default)
  KidMarital ~~ KidMarital; MomMarital ~~ MomMarital; DadMarital ~~ DadMarital
# All possible pairwise residual covariances (not estimated by default)
  KidMarital ~~ MomMarital + DadMarital; MomMarital ~~ DadMarital
# All intercepts estimated separately (by default)
  KidMarital ~ 1; MomMarital ~ 1; DadMarital ~ 1
# Regressions: y outcomes ON x predictors (label to do math on later)  
  KidMarital ~ KidBoy + KidEd12
  MomMarital ~ MomEd12
  KidMarital ~ (DadEd2K)*DadEd12 
  MomMarital ~ (DadEd2M)*DadEd12 
  DadMarital ~ (DadEd2D)*DadEd12
# Getting differences in effect of DadEd for each person
  KvMDadEd := DadEd2M - DadEd2K  # Kid v. Mom: Dad Educ Effect Diff
  KvDDadEd := DadEd2D - DadEd2K  # Kid v. Dad: Dad Educ Effect Diff
  MvDDadEd := DadEd2D - DadEd2M  # Mom v. Dad: Dad Educ Effect Diff
"
print("lavaan path analysis model estimated with ML on wide-format data")
PathModel3 = lavaan(data=Example5_wide, model=Syntax3, estimator="MLR", mimic="mplus")
summary(PathModel3, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE, ci=TRUE)

print("Request sorted modification indices for p<.05 to troubleshoot local misfit")
modindices(object=PathModel3, sort=TRUE, minimum.value=3.84)

print("Request residual covariance matrix =leftover from observed minus predicted")
resid(object=PathModel3, type="raw") # also type="cor" for correlation matrix

txtComment(" ") # insert blank space in output 


############################################################################################
print("R Model 4: Own + Dad + Mom Education a Predictor of Each Attitude")

# Create model syntax as separate text object
Syntax4 = "
# Residual variances estimated separately (by default)
  KidMarital ~~ KidMarital; MomMarital ~~ MomMarital; DadMarital ~~ DadMarital
# All possible pairwise residual covariances (not estimated by default)
  KidMarital ~~ MomMarital + DadMarital; MomMarital ~~ DadMarital
# All intercepts estimated separately (by default)
  KidMarital ~ 1; MomMarital ~ 1; DadMarital ~ 1
# Regressions: y outcomes ON x predictors (label to do math on later)  
  KidMarital ~ KidBoy + KidEd12
  KidMarital ~ (DadEd2K)*DadEd12 
  MomMarital ~ (DadEd2M)*DadEd12 
  DadMarital ~ (DadEd2D)*DadEd12
# New effects here
  KidMarital ~ (MomEd2K)*MomEd12 
  MomMarital ~ (MomEd2M)*MomEd12 
  DadMarital ~ (MomEd2D)*MomEd12
  
# Getting differences in effect of DadEd for each person
  KvMDadEd := DadEd2M - DadEd2K;  # Kid v. Mom: Dad Educ Effect Diff
  KvDDadEd := DadEd2D - DadEd2K;  # Kid v. Dad: Dad Educ Effect Diff
  MvDDadEd := DadEd2D - DadEd2M;  # Mom v. Dad: Dad Educ Effect Diff
  KvMMomEd := MomEd2M - MomEd2K;  # Kid v. Mom: Mom Educ Effect Diff
  KvDMomEd := MomEd2D - MomEd2K;  # Kid v. Dad: Mom Educ Effect Diff
  MvDMomEd := MomEd2D - MomEd2M;  # Mom v. Dad: Mom Educ Effect Diff
"
print("lavaan path analysis model estimated with ML on wide-format data")
PathModel4 = lavaan(data=Example5_wide, model=Syntax4, estimator="MLR", mimic="mplus")
summary(PathModel4, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE, ci=TRUE)

print("Request sorted modification indices for p<.05 to troubleshoot local misfit")
modindices(object=PathModel4, sort=TRUE, minimum.value=3.84)

print("Request residual correlation matrix =leftover from observed minus predicted")
resid(object=PathModel4, type="raw") # also type="cor" for correlation matrix


# Close external output file
txtStop()

